#!/usr/bin/python

# SitQuietly v1.0.1
# A Meditation timer.
# Copyright 2007 Richard Barker

# This program is distribusted under the terms of the
# GNU General Public License version 2. 

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys, time, math, os, os.path
import pygtk
pygtk.require('2.0')
import pygst
pygst.require('0.10')
import gst, gtk, gtk.glade
from threading import *

class sitquietly:
	#Display the about dialog box
	def showabout(self, widget, data=None):
		wDlg = gtk.glade.XML(self.gladefile, "aboutdialog1")
		dlgAbout = wDlg.get_widget("aboutdialog1")
		result =dlgAbout.run()
		dlgAbout.hide()
		
	#Display message dialog boxes
	def showdialog(self,title, text, type):
		if type == 0: 	#OK / CANCEL
			buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)
		elif type == 1: #Simple OK
			buttons = (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT)		
		dialog = gtk.Dialog(title,self.window,gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,buttons)
		label = gtk.Label(text)
		dialog.vbox.pack_start(label, False, False, 0)
		label.show()
		result = dialog.run()
		dialog.hide()
		return(result)

	#Display the online help.
	def showhelp(self,widget):
		try:
			os.spawnlp(os.P_NOWAIT,"yelp","yelp","/usr/share/sitquietly/sqhelp.xml")
		except:
			self.showdialog("Error","Unable to display online help.",1)

	#Play the bell sample
	def playbell(self):
		self.player.set_state(gst.STATE_NULL)
		self.player.set_state(gst.STATE_PLAYING)
		return False
	
	#Run the main timer
	def rungui(self, data=None):
		self.count=self.count+1							#Increment the main counter
		done = 1.0*self.count/self.tlength			
		self.progressbar1.set_fraction(done)		#Update the progress bar
		if self.count < self.tlength:			
			return True											#Return true if we're not done
		else:
			self.playbell()										#Else, play the bell.
			self.progstate = 2								#Set state to stopped
			self.progressbar1.set_fraction(0)		#Reset the progress bar
			self.button1.set_sensitive(True)			#Enable / disable relevant GUI controls
			self.button2.set_sensitive(False)
			self.button3.set_sensitive(False)
			self.spinbutton1.set_sensitive(True)
			self.spinbutton2.set_sensitive(True)
			return False										#Return false, stopping the thread respawn.

	#Start the meditation sitting.
	def start(self, widget, data=None):
			if self.progstate > 0:							#Only start if we're stopped or paused.
				self.intime = self.spinbutton1.get_value_as_int()
				time = self.spinbutton2.get_value_as_int()
				time = time*60									# Convert the meditation time to seconds.
				if time >= self.intime:						#Intro must be shorter than the sitting.
					if self.progstate == 2:					#If we're stopped, not paused, reset the data
						self.tlength = time+self.intime		#Set total length, then reset the counter.
						self.count=0							
					if self.count <= self.intime and self.intime >0:	#Set the intro timer, if needed.
						self.t1 = gtk.timeout_add(1000*(self.intime-self.count),self.playbell)
					self.t3 = gtk.timeout_add(1000, self.rungui)	#Set the main timer.
					self.progstate=0										#Set sate to playing.
					self.button1.set_sensitive(False)				#Toggle GUI element activation.
					self.button2.set_sensitive(True)
					self.button3.set_sensitive(True)
					self.spinbutton1.set_sensitive(False)
					self.spinbutton2.set_sensitive(False)
				else:
					self.showdialog("Error","Settle time is longer than sitting.",1)  #Oops, PEBKAC!
					
	#Pause the sitting			
	def pause(self, widget, data=None):
		#We can only pause if playing
		if self.progstate == 0:
			if self.intime >0:
				gtk.timeout_remove(self.t1)			#Remove timers
			gtk.timeout_remove(self.t3)
			self.progstate = 1							#Set state to paused
			self.button1.set_sensitive(True)		#Toggle GUI element activation.
			self.button2.set_sensitive(False)
			self.button3.set_sensitive(True)
			
	#Stop the sitting.	
	def stop(self, widget, data=None):
		#We can only stop if playing or paused.
		if self.progstate < 2:
			if self.intime > 0:
				gtk.timeout_remove(self.t1)			#Remove the timers
			gtk.timeout_remove(self.t3)
			self.progstate = 2							#Set state to stopped
			self.progressbar1.set_fraction(0)	#Clear the progressbar
			self.button1.set_sensitive(True)		#Toggle GUI element activation.
			self.button2.set_sensitive(False)
			self.button3.set_sensitive(False)
			self.spinbutton1.set_sensitive(True)
			self.spinbutton2.set_sensitive(True)
			
	#GTK event handler, someone hit the quit button.	
	def delete_event(self, widget, data=None):
		result = self.showdialog("Confirm Exit","Are you sure you want to exit?",0)  #Confirm dialog
		if result == gtk.RESPONSE_REJECT:		
			return True										#No, don't quit
		else:
			intime = self.spinbutton1.get_value_as_int()		#get the timer settings
			time = self.spinbutton2.get_value_as_int()
			try:
				file = open(self.rcfilename,"w")						#Write them to the 'rc' file
				file.write(str(intime)+","+str(time))
				file.close()
			except:
				self.showdialog("Error","Unable to store timer settings.",1)		#Error writing
			gtk.main_quit()			
			return False									#Quit.
		
	#GTK event handler, destory the main window.
	def destroy(self, widget, data=None):
		gtk.main_quit()
		
	# Gstreamer event handler (see below)    
	def on_message(self, bus, message):
		t = message.type
		if t == gst.MESSAGE_EOS:						#End of stream, stop playing.
				self.player.set_state(gst.STATE_NULL)
		elif t == gst.MESSAGE_ERROR:				#Error playing the bell, scream at user.
			result = showdialog("GStreamer Error","Unable to play sound!",1)
			self.player.set_state(gst.STATE_NULL)
			
	# GStreamer demuxer callback
	def demuxer_callback(self, demuxer, pad):
		dec_pad = self.audio_decoder.get_pad("sink")
		pad.link(dec_pad)

	#Initialise the class.
	def __init__(self):
		self.progstate=2
		self.rcfilename = os.path.expanduser("~/.sitquietlyrc")
		self.soundfile = "/usr/share/sitquietly/2166_suburban_grilla_bowl_struck.ogg"
		self.gladefile= "/usr/share/sitquietly/sitquietly.glade"

	# The GStreamer audio pipeline
		#Create a gst playbin, set the output sink and the input file.
		self.player = gst.Pipeline("player")
		size = os.path.getsize(self.soundfile)
		source = gst.element_factory_make("filesrc", "file-source")
		self.player.add(source)
		self.player.get_by_name("file-source").set_property('location', self.soundfile)
		self.player.get_by_name("file-source").set_property('mmapsize', size)	#Precache sample.
		demuxer = gst.element_factory_make("oggdemux", "demuxer")
		self.player.add(demuxer)
		demuxer.connect("pad-added", self.demuxer_callback)
		self.audio_decoder = gst.element_factory_make("vorbisdec", "vorbis-decoder")
		self.player.add(self.audio_decoder)
		audioconv = gst.element_factory_make("audioconvert", "converter")
		self.player.add(audioconv)
		audiosink = gst.element_factory_make("autoaudiosink", "audio-output")
		self.player.add(audiosink)
		gst.element_link_many(source, demuxer)
		gst.element_link_many(self.audio_decoder, audioconv, audiosink)
		bus = self.player.get_bus()
		bus.add_signal_watch()
		bus.connect('message', self.on_message)
		
		# Load the main window from Glade XML file.
		self.wTree = gtk.glade.XML(self.gladefile, "winMain")

		#Create our dictionary and connect it
		dic = { "on_button1_clicked" : self.start,
					"on_button2_clicked" : self.pause,
					"on_button3_clicked" : self.stop,
					"on_quit1_activate" : self.delete_event,
					"on_help1_activate":self.showhelp,
					"on_about1_activate" : self.showabout,
					"on_winMain_delete_event": self.delete_event,
					"on_winMain_destroy_event" : self.destroy }
		self.wTree.signal_autoconnect(dic)
		
		#Get handles to the GUI elements
		self.window = self.wTree.get_widget("winMain")
		self.progressbar1 = self.wTree.get_widget("progressbar1")
		self.spinbutton1 = self.wTree.get_widget("spinbutton1")
		self.spinbutton2 = self.wTree.get_widget("spinbutton2")
		self.button1 = self.wTree.get_widget("button1")
		self.button2 = self.wTree.get_widget("button2")
		self.button3 = self.wTree.get_widget("button3")

		try:	
			file = open(self.rcfilename,"r")						#Open the rc file.
			rcdata = file.read()										#Read it
			times = rcdata.split(",")								#Split the data, set the timers
			self.spinbutton1.set_value(int(times[0]))
			self.spinbutton2.set_value(int(times[1]))
			file.close()
		except:
			self.showdialog("Error","Unable to read timer settings, setting defaults. \nIf this is the first time Sitquietly has run, please \ndisregard this message",1)		#Problem.

	def main(self):
		gtk.main()

if __name__ == "__main__":
	sit = sitquietly()
	sit.main()